package org.example;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import org.example.BeatThemAllGame.Item;
import org.example.BeatThemAllGame.Logger;

// Classe pour le héros
public class Hero extends Character {
    boolean specialUsed = false;
    private List<Item> inventory = new ArrayList<>();
    private int experience = 0;
    private int level = 1;

    public Hero(String name, int maxHP, int attackPower, int defense, SpecialAbility specialAbility) {
        super(name, maxHP, attackPower, defense);
        this.specialAbility = specialAbility;
    }

    @Override
    public void attack(Character target) {
        System.out.println(name + " attaque " + target.name + " !");
        Logger.getInstance().log(name + " attaque " + target.name);
        target.takeDamage(attackPower);
    }

    public void useSpecialAbility(Character target) {
        if (!specialUsed) {
            specialAbility.activate(this, target);
            specialUsed = true;
            Logger.getInstance().log(name + " utilise sa capacité spéciale");
        } else {
            System.out.println("Capacité spéciale déjà utilisée !");
        }
    }

    public void resetSpecialAbility() {
        specialUsed = false;
    }

    public void addItem(Item item) {
        inventory.add(item);
        System.out.println(item.getName() + " a été ajouté à votre inventaire.");
        Logger.getInstance().log(name + " a obtenu un objet : " + item.getName());
    }

    public void useItem() {
        if (inventory.isEmpty()) {
            System.out.println("Votre inventaire est vide !");
            return;
        }
        System.out.println("Inventaire :");
        for (int i = 0; i < inventory.size(); i++) {
            System.out.println((i + 1) + ". " + inventory.get(i).getName());
        }
        System.out.print("Choisissez un objet à utiliser : ");
        Scanner scanner = new Scanner(System.in);
        int choice = scanner.nextInt();
        if (choice > 0 && choice <= inventory.size()) {
            Item item = inventory.get(choice - 1);
            item.applyEffect(this);
            inventory.remove(item);
            Logger.getInstance().log(name + " utilise un objet : " + item.getName());
        } else {
            System.out.println("Choix invalide.");
        }
    }

    public void gainExperience(int xp) {
        experience += xp;
        System.out.println(name + " gagne " + xp + " points d'expérience.");
        Logger.getInstance().log(name + " gagne " + xp + " XP");
        if (experience >= level * 100) {
            levelUp();
        }
    }

    private void levelUp() {
        level++;
        experience = 0;
        maxHP += 20;
        attackPower += 5;
        defense += 2;
        currentHP = maxHP;
        System.out.println(name + " passe au niveau " + level + " !");
        Logger.getInstance().log(name + " monte au niveau " + level);
    }
}
